#!/bin/bash
# called by user to perform the necessary updation
# perform the process of updating the files in the safesquid folder
# ACTION(s) performed : softlink, mkdir, copy with overwite, copy without overwrite
# OWNERSHIP and PERMISSIONS will also be set

LIST="list.conf";
LOGF="/tmp/setup.log";
DATE=`date`;
L=0;

SRC_DIR="/etc/openvpn"
LNK_DIR="/usr/local/safesquid/security/openvpn"
_RSYNC=/usr/bin/rsync
APPLIANCE="/tmp/safesquid/_mkappliance"

LOG()
{
	echo "${DATE} - ${*}" 
}

INSTALL_PACKAGES()
{	
	PACKS=`<./dependencies.lst`
	
	if [ "${PACKS}x" != "x" ]; then
		export DEBIAN_FRONTEND=noninteractive
		aptitude -q -y install ${PACKS}
		export DEBIAN_FRONTEND=
	fi	
}

SET_IPTABLES(){
#substitute for ufw 
## Setting default policies:
#iptables -P INPUT DROP
#iptables -P FORWARD DROP
#iptables -P OUTPUT ACCEPT
#
## Exceptions to default policy
#iptables -A INPUT -p tcp --dport 22 -j ACCEPT       # SSH
#iptables -A INPUT -p tcp --dport 53 -j ACCEPT       # bind service
#iptables -A INPUT -p tcp --dport 80 -j ACCEPT       # HTTP
#iptables -A INPUT -p tcp --dport 443 -j ACCEPT      # HTTPS
#iptables -A INPUT -p udp --dport 1194 -j ACCEPT     # Openvpn
#iptables -A INPUT -p tcp --dport 8080 -j ACCEPT     # SafeSquid 
#iptables -A INPUT -p tcp --dport 8443 -j ACCEPT     # SafeSquid
#iptables -A PREROUTING -p tcp -m tcp --dport 80 -j REDIRECT --to-ports 8080 # redirect from 80 to 8080
#iptables -A PREROUTING -p tcp -m tcp --dport 443 -j REDIRECT --to-ports 8443 # redirect from 443 to 8443
#iptables -A POSTROUTING -s 10.0.0.0/8 -o eth0 -j MASQUERADE # openvpn masquerade

CHK_80=$( cat /etc/iptables/rules.v4 | grep '80 -j REDIRECT --to-ports 8081' |wc -l )
CHK_443=$( cat /etc/iptables/rules.v4 | grep '443 -j REDIRECT --to-ports 8443' |wc -l)
CHK_MSQ=$( cat /etc/iptables/rules.v4 | grep 'eth0 -j MASQUERADE' |wc -l )

	if [ "x${CHK_80}" == "x0" ] ;then
	iptables -t nat -A PREROUTING  -p tcp -m tcp --dport 80 -j REDIRECT --to-ports 8081
	LOG "IPTABLES : redirect 80 to 8081"
	fi

	if [ "x${CHK_443}" == "x0" ] ;then
	iptables -t nat -A PREROUTING  -p tcp -m tcp --dport 443 -j REDIRECT --to-ports 8443
	LOG "IPTABLES : redirect 443 to 8443"
	fi	

	if [ "x${CHK_MSQ}" == "x0" ] ;then
	iptables -t nat -A POSTROUTING -s 10.0.0.0/8 -o eth0 -j MASQUERADE	
	LOG "IPTABLES : postrouting"
	fi

	iptables-save > /etc/iptables/rules.v4
}

SET_UFW(){
	ufw allow ssh
	ufw allow 80/tcp
	ufw allow 443/tcp
	ufw allow 1194/udp
	ufw allow 8080
	ufw allow 8081
	ufw allow 8443
	ufw allow 53/udp
	
	sed -i 's/DEFAULT_FORWARD_POLICY="DROP"/DEFAULT_FORWARD_POLICY="ACCEPT"/g' /etc/default/ufw	
	[ ! -f "/etc/ufw/before.rules_default" ] && cp /etc/ufw/before.rules  /etc/ufw/before.rules_default
	
	awk 'NR<10' /etc/ufw/before.rules_default > /etc/ufw/before.rules_new	
cat << _EOF >>  /etc/ufw/before.rules_new
# START OPENVPN RULES
# NAT table rules
*nat
:POSTROUTING ACCEPT [0:0] 
# Allow traffic from OpenVPN client to eth0
-A POSTROUTING -s 10.8.0.0/8 -o eth0 -j MASQUERADE
COMMIT
# END OPENVPN RULES
_EOF

	awk 'NR>10' /etc/ufw/before.rules_default >> /etc/ufw/before.rules_new	
	cat /etc/ufw/before.rules_new > /etc/ufw/before.rules	
	ufw disable
	echo 'y' | ufw enable

}

OPEN_VPN_SETUP(){
	
	echo 1 > /proc/sys/net/ipv4/ip_forward
	
	[ ! -d ${SRC_DIR} ] && mkdir ${SRC_DIR}
	chmod 775 ${SRC_DIR}
	
	[ ! -L ${LNK_DIR} ] && ln -fs ${SRC_DIR} ${LNK_DIR} 
	chown -R ssquid:root /usr/local/safesquid/security/openvpn
	[ ! -f "/etc/openvpn/openvpn-connection.log" ] && touch /etc/openvpn/openvpn-connection.log 
	chmod -R 777 /etc/openvpn/openvpn-connection.log
#	/etc/init.d/monit restart	
	
	LOG "please wait .. generating certifiactes for openvpn"
	openssl dhparam -out /etc/openvpn/dh2048.pem 2048 2> /dev/null
	

	
}

MAIN()
{
	INSTALL_PACKAGES
	SET_IPTABLES
	SET_UFW
	
	${_RSYNC} -apv ${APPLIANCE}/* / 2> ${LOGF}; ret=$?
	[ "x${ret}" == "x1" ] && LOG "rsync: failed to sync ${APPLIANCE} into system\n" && return;
	
	OPEN_VPN_SETUP
}


> ${LOGF};

MAIN 2>&1 | tee -a ${LOGF};
